#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#define OLED_RESET 4
#define SCREEN_WIDTH 128
#define SCREEN_HEIGHT 64
#define BUTTON_PIN 2
#define RED_PIN 5
#define GREEN_PIN 6
#define BLUE_PIN 7
#define BUZZER_PIN 9

int TIME_UNTIL_WARMUP = 10;
unsigned long time;
unsigned long startTime; // Variable to store the start time of alcohol level measurement

int analogPin = A0; // Changed to A0 for analog input
int val = 0;
bool buttonPressed = false; // Variable to track button press
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET); // Added SCREEN_WIDTH and SCREEN_HEIGHT
int buttonState = HIGH;  // Current state of the button
int lastButtonState = LOW; // Previous state of the button
bool startMeasuring = false; // Variable to start the measure
bool finalValue = false; // Variable to display the las value measured
bool resultPrinted = false; // Variable to track if result has been printed

void setup() {
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);
  display.clearDisplay();
  //button
  pinMode(BUTTON_PIN, INPUT_PULLUP);
  //led
  pinMode(RED_PIN, OUTPUT);
  pinMode(GREEN_PIN, OUTPUT);
  pinMode(BLUE_PIN, OUTPUT);
  //buzzer
  pinMode(BUZZER_PIN, OUTPUT);
  digitalWrite(BUZZER_PIN, LOW);
}

void loop() {
  delay(100);

  printTitle();

  //Led off
  digitalWrite(RED_PIN, HIGH);
  digitalWrite(GREEN_PIN, HIGH);
  digitalWrite(BLUE_PIN, HIGH);


  // Read the state of the button
  buttonState = digitalRead(BUTTON_PIN);

  // Check if the button state has changed
  if (buttonState != lastButtonState) {
    // Check if the button is pressed
    if (buttonState == LOW) {
      // Button is pressed, set buttonPressed to true
      buttonPressed = true;
      startMeasuring = true;
      finalValue = false;
      startTime = millis(); // Record the start time of alcohol level measurement
      noTone(BUZZER_PIN);
    } else {
      // Button is released, set buttonPressed to false
      buttonPressed = false;
    }
    // Save the current button state for comparison in the next iteration
    lastButtonState = buttonState;
  }

  // Display warming up message until TIME_UNTIL_WARMUP is reached
  time = millis() / 1000;
  if (time <= TIME_UNTIL_WARMUP) {
    printWarming();
    time = map(time, 0, TIME_UNTIL_WARMUP, 0, 100);
    display.drawRect(10, 50, 110, 10, WHITE); //Empty Bar
    display.fillRect(10, 50, time, 10, WHITE);
  } else {
    // If warming up time is over, display alcohol level
    if (startMeasuring) {
      if ((millis() - startTime) < 7000) { // Check if 7 seconds have not passed since the start of measurement
        val = readAlcohol();
        printTitle();
        printAlcohol(val);
        printAlcoholLevel(val);
      } else {
        // If 7 seconds have passed, stop measuring
        startMeasuring = false;
        // Display the last measured alcohol level
        finalValue = true;
      }
    } else {
      // If button is not pressed or 7 seconds have passed, display button prompt
      if (!finalValue) {
        printButtonPrompt();
      }
    }
  }

  if(finalValue) {
    printAlcohol(val);
    printAlcoholLevel(val);
    printResult(val);
  }

  display.display();
}

void printResult(int value) {
  if (value < 200) {
    display.println("You can drive.");
    // Turn on green
    digitalWrite(RED_PIN, HIGH);
    digitalWrite(GREEN_PIN, LOW);
    digitalWrite(BLUE_PIN, HIGH);

  }
  if (value >= 200 && value < 280) {
    display.println("Wait one hour.");
    // Turn on yellow
    digitalWrite(RED_PIN, LOW);
    digitalWrite(GREEN_PIN, LOW);
    digitalWrite(BLUE_PIN, HIGH);
    //buzzer sound
    tone(BUZZER_PIN, 587); // A4
    delay(1000);
    noTone(BUZZER_PIN); 

  }
  if (value >= 280 && value < 350) {
    display.println("Wait a few hours.");
    // Turn on yellow
    digitalWrite(RED_PIN, LOW);
    digitalWrite(GREEN_PIN, LOW);
    digitalWrite(BLUE_PIN, HIGH);
    //buzzer sound
    tone(BUZZER_PIN, 587); // A4
    delay(1000);
    noTone(BUZZER_PIN); 
  }
  if (value >= 350) {
    display.println("Do not drive!");
    // Turn on red
    digitalWrite(RED_PIN, LOW);
    digitalWrite(GREEN_PIN, HIGH);
    digitalWrite(BLUE_PIN, HIGH);
    //buzzer sound
    tone(BUZZER_PIN, 784); // G4
  }
}

void printButtonPrompt() {
  // display.clearDisplay(); // Clear the display before drawing
  display.setTextSize(1);
  display.setTextColor(WHITE);

  // Text to be displayed
  const char* textReady = "Ready";
  const char* textPrompt = "Press button to start";

  // Calculate horizontal positions
  int16_t readyX = (SCREEN_WIDTH - (strlen(textReady) * 6)) / 2;
  int16_t readyY = SCREEN_HEIGHT / 3;

  int16_t promptX = (SCREEN_WIDTH - (strlen(textPrompt) * 6)) / 2;
  int16_t promptY = readyY + 10; // Adjust the vertical spacing as needed

  // Draw "Ready" centered
  display.setCursor(readyX, readyY);
  display.println(textReady);

  // Draw "Press button to start" centered below "Ready"
  display.setCursor(promptX, promptY);
  display.println(textPrompt);

  // display.display(); // Update the display with the new text
}


void printTitle() {
  display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor((SCREEN_WIDTH - 60) / 2, 0); // Centered horizontally
  display.println("Breathalyzer");
}

void printWarming() {
  display.setTextSize(2);
  display.setTextColor(WHITE);
  display.setCursor(15, SCREEN_HEIGHT / 4); // Centered vertically
  display.println("Starting");
}

void printAlcohol(int value) {
  display.setTextSize(2);
  display.setTextColor(WHITE);
  display.setCursor((SCREEN_WIDTH - 48) / 2, SCREEN_HEIGHT / 3); // Centered horizontally and vertically
  display.println(val);
}

void printAlcoholLevel(int value) {
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(10, SCREEN_HEIGHT / 2 + 20); // Centered vertically
}

int readAlcohol() {
  int val = 0;
  int val1;
  int val2;
  int val3;

  display.clearDisplay();
  val1 = analogRead(analogPin);
  delay(10);
  val2 = analogRead(analogPin);
  delay(10);
  val3 = analogRead(analogPin);

  val = (val1 + val2 + val3) / 3;
  return val;
}